import '@host';
import './generate.scss';

import { Component }   from '../../scripts/decorators/component';
import { View }        from '../../scripts/decorators/view';
import { inject }      from '../../scripts/decorators/inject';
import { Generate }    from './generate';

@Component({
  selector: 'generate'
})

@View({
  template: require('./generate.html')
})

/**
 * @class GenerateComponent
 */
export class GenerateComponent {
  @inject
  private generate: Generate;

  @inject
  private $q: angular.IQService;

  // TODO: interface for loopback models
  models: { [name: string]: any } = {};
  modelFields: Array<AngularFormly.IFieldConfigurationObject>;

  // TODO: interface for loopback model
  activeModel: any;
  values: any;

  constructor() {
    this.getModels();
  }

  loadModel(name: string) {
    this.activeModel = this.models[name];
    this.values = {};
    this.modelFields = this.generate.modelToFormDefinition(this.activeModel);
  }

  getModels() {
    // get list of models
    this
      .generate
      .getModels()
      .then(models => {
        const promises: Array<angular.IPromise<any>> = [];

        // for each model get the model JSON value
        models.forEach(model => {
          const promise = this
            .generate
            .getModel(model)
            .then(response => {
              this.models[model] = response.data;
            });

          promises.push(promise);
        });

        // When all promises are complete set the first model as the active one
        this
          .$q
          .all(promises)
          .then(() => {
            for (const key in this.models) {
              this.loadModel(key);
              break;
            }
          });
      });
  }
}
