import { Service }  from '../../scripts/decorators/service';
import { inject }   from '../../scripts/decorators/inject';

import 'angular-formly';
import 'angular-formly-templates-ionic';

interface ILoopbackFile {
  container: string;
  name:      string;
  size:      number;
  atime:     string;
  mtime:     string;
  ctime:     string;
}

@Service('generate')

/**
 * Generate
 */
export class Generate {
  @inject
  private $http: angular.IHttpService;

  modelToFormDefinition(model: any) {
    const fields: AngularFormly.IFieldConfigurationObject[] = [];

    const typeMap = {
      string: 'input',
      boolean: 'toggle',
      number: 'input'
    };

    const props = model.properties;
    for (const key in props) {
      let type = props[key].type;
      fields.push({
        key: key,
        type: typeMap[type],
        templateOptions: {
          label: _.startCase(key),
          placeholder: type
        }
      });
    }

    return fields;
  }

  getModels() {
    return this
      .$http
      .get('/api/v1/localFiles/models/files')
      .then((response: angular.IHttpPromiseCallbackArg<ILoopbackFile[]>) => {
        const data = response.data;

        return data
          .filter(file => {
            return !Boolean(file.name.match(/\.js$|local-files/));
          })
          .map(file => {
            return _.camelCase(file.name.replace(/\.json$/, ''));
          });
      });
  }

  getModel(name: string) {
    return this
      .$http
      .get(`/api/v1/localFiles/models/download/${encodeURIComponent(_.kebabCase(name))}.json`);
  }
}
