import '@host';
import './login.scss';

import { HotReload } from '../../scripts/decorators/hot-reload';
import { Component } from '../../scripts/decorators/component';
import { View }      from '../../scripts/decorators/view';
import { inject }    from '../../scripts/decorators/inject';
import { User }      from '../../scripts/services/user/user';

// directives
import './error-messages/error-messages-component';
import './form-field/form-field-component';
import './disclosure/disclosure-component';
import './social-auth-buttons/social-auth-buttons-component';

@HotReload(module)

@Component({
  selector: 'login',
  bind: {
    socialAuth: '=',
    method: '@',
    onSuccess: '&',
    onError: '&'
  }
})

@View({
  template: require('./login.html')
})

/**
 * @class LoginComponent
 */
export class LoginComponent {
  @inject
  private $ionicLoading: ionic.loading.IonicLoadingService;

  @inject
  private $ionicPopup: ionic.popup.IonicPopupService;

  @inject
  user: User;

  usernamePlaceholder: string;
  method: string = angular.isDefined(this.method) ? this.method : 'signup';
  socialAuth: boolean;
  username: string;
  password: string;
  rejection: any;

  onSuccess: any;
  onError: any;

  constructor() {
    this.usernamePlaceholder = this.method === 'signup' ? 'Email' : 'Username or Email';
    this.username = this.user.data && (this.user.data.handle || this.user.data.email);
  }

  switchMethod(state: string) {
    this.method = state;
    this.usernamePlaceholder = state === 'login' ? 'Username or Email' : 'Email';
  }

  auth(authMethod: string) {
    this.$ionicLoading.show();
    const method = authMethod || this.method;
    const loginCall = (method === 'signup' || method === 'login') ?
        this.user[method](this.username, this.password) :
        this.user[method]();

    loginCall
      .then(this.authSuccess.bind(this))
      .catch(this.authFailure.bind(this))
      .finally(() => {
        this.$ionicLoading.hide();
      });
  }

  private authSuccess(...args: any[]) {
    if (this.onSuccess) {
      this.onSuccess.apply(null, args);
    }
  }

  private authFailure(...args: any[]) {
    this.rejection = args[0].data || args[0];

    if (!this.rejection.errorMessage) {
      this.rejection.errorMessage = 'An unexpected error occured.';
    }

    if (this.onError) {
      this.onError.apply(null, args);
    }
  }

  showAlert(title: string) {
    this.$ionicPopup.alert({
      title: title
    });
  }

  forgotPassword() {
    // Request Method:POST
    this.$ionicPopup.prompt({
      title: 'Forgot Password',
      template: 'We\'ll send you a link to reset your password',
      inputType: 'email',
      inputPlaceholder: 'Email Address',
      cssClass: 'forgot-password'
    })
    .then((email: string) => {
      if (email) {
        this
          .user
          .forgotPassword(email)
          .catch( (response) => {
            if (response.status >= 400) {
              this.showAlert(response.data.errorMessage);
            }
          });
      }
    });
  }
}
