import { Constant } from '../../decorators/constant';

@Constant('numberUtils', true)

/**
 * @class NumberUtils
 *
 * Utils for numbers
 */
export class NumberUtils {
  /**
   * Pretty format a number.
   *
   * @example
   * numberUtils.prettyFormat(5100000);
   * // returns '5.1M'
   *
   * @param  {number} value Number to convert
   * @return {string}       String of converted number
   */
  prettyFormat(value: number): string {
    value = Math.abs(value);

    if (value >= 1000000000000) {
      return (value / 1000000000000).toFixed(1) + 'T';
    } else if (value >= 1000000000) {
      return (value / 1000000000).toFixed(1) + 'B';
    } else if (value >= 1000000) {
      return (value / 1000000).toFixed(1) + 'M';
    } else if (value >= 1000) {
      return (value / 1000).toFixed(1) + 'K';
    } else if (value < 1 && value > 0) {
      return value.toFixed(2);
    } else if (value === 0) {
      return '0';
    } else if (value) {
      return value.toFixed(0);
    } else {
      return '';
    }
  }
}
