import { makeInjectable } from './inject';

/**
 * @decorator Run
 *
 * @type {ClassDecorator}
 *
 * @example
 * ```ts
 * @Run
 * export class Routes {
 * 	 @inject
 * 	 private $stateProvider: angular.ui.IStateProvider;
 *
 * 		// Constructor takes no args for config
 * 		// classes
 * 		constructor() {
 * 			this.$stateProvider.state({
 * 			  // ...
 * 			});
 * 		}
 * }
 * ```
 */
export const Run: ClassDecorator = (Target: any) => {
  const target = makeInjectable(Target, false);

  // Construct the module
  angular
    .module('app')
    .run(target);
};
