import { makeInjectable } from './inject';

const serviceCache = {};

/**
 * @decorator Service
 *
 * @description create angular services
 *
 * @param {string} name
 * name of your service
 *
 * @example
 * ```ts
 * @Service('abTests')
 * export class AbTests {
 *   @inject str: Str;
 *
 * 	 onstructor() {
 * 	   // do something...
 * 	   str.dasherize('foobar');
 * 	 }
 * }
 * ```
 */
export const Service = (name: string) => {
  const decorator: ClassDecorator = (Target: any) => {
    const target = makeInjectable(Target);

    // TODO: do with constants, etc
    if (serviceCache[name]) {
      const injector = angular.element(document).injector();

      const invoked = injector.invoke(target);
      const original = injector.get(name);

      for (const key in original) {
        delete original[key];
      }

      // TODO: does not handle deletions - maybe wipe out the original
      // object first
      for (const key in invoked) {
        original[key] = invoked[key];
      }
    } else {
      // Construct the module
      angular
        .module('app')
        .service(name, target);

      serviceCache[name] = target;
    }
  };

  return decorator;
};
