import { Directive } from '../decorators/directive';

@Directive('active')

/**
 * @directive active
 *
 * @todo base class to inherit
 *
 * @description
 * Similar to ng-show and ng-hide, toggles .active and .not-active classes
 * based on argument
 *
 * @example
 * ```html
 * <div active="activeTab === 'foobar'"></div>
 * ```
 */
export class ActiveDirective implements angular.IDirective {

  // FIXME: why are the argument types not being pulled by angular.IDirective?
  // it's all already defined in the IDirective interface in angular.d.ts
  link(scope: ng.IScope, element: ng.IAugmentedJQuery, attrs: any) {

    scope.$watch(attrs.active, newVal => {
      if (newVal) {
        element
          .addClass('active')
          .removeClass('not-active');
      } else {
        element
          .addClass('not-active')
          .removeClass('active');
      }
    });
  }
}
