import '../dependencies/app';

import { inject }       from '../../scripts/decorators/inject';
import { UrlUtils }     from '../../scripts/constants/utils/url';
import { Interceptor }  from '../../scripts/decorators/interceptor';

const DigestGenerator = require('@ssjs/node-shopstyle-auth-digest-generator');
const digestGenerator = new DigestGenerator();

@Interceptor
export class AuthorizationInterceptor {

  @inject
  $cookies: angular.cookies.ICookiesService;

  @inject
  $location: angular.ILocationService;

  @inject
  $q: any;

  @inject
  urlUtils: UrlUtils;

  @inject
  config: any;

  request(requestConfig: angular.IRequestConfig) {
    const user = this.$cookies.getObject('userData');

    if (this.isApiCall(requestConfig.url)) {
      requestConfig.params = requestConfig.params || {};

      // put any default params (like pid) in config.defaultRequestParams
      _.defaults(requestConfig.params, this.config.defaultRequestParams);

      const method = requestConfig.method;

      // Remove query parameters from the url
      if (requestConfig.url.indexOf('?') > -1) {
        // parse existing params from requestConfig.url
        _.assign(requestConfig.params, this.urlUtils.getQueryParams(requestConfig.url));

        // remove existing params from requestConfig.url
        requestConfig.url = requestConfig.url.split('?')[0];
      }

      // add debugging params if the debug flag is set in config
      if (this.config.authDebug) {
        requestConfig.params.authDebug = 1;
        requestConfig.params.includeStackTrace = 1;
      }

      // build the final URL for our digest
      let url = this.urlUtils.addParams(requestConfig.url, requestConfig.params);
      const a = document.createElement('a');
      a.href = url;
      url = a.pathname + a.search;

      let body: any;
      if (requestConfig.data) {
        body = JSON.parse(angular.toJson(requestConfig.data));
      }

      if (user && user.id) {
        const loginTimestamp = user.loginTimestamp;
        const secret = user.loginToken;
        const now = new Date();

        const request = digestGenerator.createRequest(method, url, body);
        const digest = digestGenerator.generateDigest(request, now.toUTCString(), secret);
        requestConfig.headers.Authorization = 'PopSugar userId=' + user.id + ', ' +
        'loginTimestamp=' + loginTimestamp + ', digest=' + digest +
        ', version=0';
        requestConfig.headers['Authorization-Date'] = now.toUTCString();
      }
      return requestConfig;
    }
    return requestConfig;
  };

  responseError(rejection: any) {
    if (rejection.status === 401) {
      this.$cookies.remove('userData');
      this.$location.path('/');
    }

    return this.$q((resolve: any, reject: any) => {
      reject(rejection);
    });
  };

  private isApiCall(url: any) {
    // if the request is relative (starts with a //), or if it contains
    // 'shopstyle', then we know it's an API request and should be authenticated
    const apiRegex = /^\/\/|shopstyle/;
    return apiRegex.test(url);
  }
}
